@@ -65,9 +65,9 @@ class TwilioRestClient(object):
                 'HTTP %s method not implemented' % method)
 
         if path[0] == '/':
-            uri = _TWILIO_API_URL + path
+            uri = self.base + path
         else:
-            uri = _TWILIO_API_URL + '/' + path
+            uri = self.base + '/' + path
 
         if method == "GET":
             params = vars
@@ -93,8 +93,23 @@ class TwilioRestClient(object):
         if not account or not token:
             account, token = find_credentials()
             if not account or not token:
-                raise TwilioException("Could not find account credentials")
+                raise TwilioException("""
+Twilio could not find your account credentials. Pass them into the
+TwilioRestClient constructor like this:
 
+    client = TwilioRestClient(account='AC38135355602040856210245275870',
+                              token='2flnf5tdp7so0lmfdu3d')
+
+Or, add your credentials to your shell environment. From the terminal, run
+
+    echo "export TWILIO_ACCOUNT_SID=AC3813535560204085626521" >> ~/.bashrc
+    echo "export TWILIO_AUTH_TOKEN=2flnf5tdp7so0lmfdu3d7wod" >> ~/.bashrc
+
+and be sure to replace the values for the Account SID and auth token with the
+values from your Twilio Account at https://www.twilio.com/user/account.
+""")
+        
+        self.base = base
         auth = (account, token)
         version_uri = "%s/%s" % (base, version)
         account_uri = "%s/%s/Accounts/%s" % (base, version, account)
@@ -119,7 +134,7 @@ class TwilioRestClient(object):
     def participants(self, conference_sid):
         """
         Return a :class:`Participants` instance for the :class:`Conference`
-        with conference_sid,
+        with the given conference_sid
         """
         base_uri = "%s/Conferences/%s" % (self.account_uri, conference_sid)
         return Participants(base_uri, self.auth)
