@@ -1,3 +1,4 @@
+import re
 import datetime
 import logging
 import twilio
@@ -8,10 +9,10 @@ from urlparse import urlparse
 
 # import json
 try:
-    import simplejson as json
+    import json
 except ImportError:
     try:
-        import json
+        import simplejson as json
     except ImportError:
         from django.utils import simplejson as json
 
@@ -92,6 +93,8 @@ def normalize_dates(myfunc):
             if len(res):
                 kwargs[k] = parse_date(v)
         return myfunc(*args, **kwargs)
+    inner_func.__doc__ = myfunc.__doc__
+    inner_func.__repr__ = myfunc.__repr__
     return inner_func
 
 
@@ -99,7 +102,6 @@ class Response(object):
     """
     Take a httplib2 response and turn it into a requests response
     """
-
     def __init__(self, httplib_resp, content, url):
         self.content = content
         self.cached = False
@@ -124,7 +126,13 @@ def make_request(method, url,
         http.add_credentials(auth[0], auth[1])
 
     if data is not None:
-        data = urlencode(data)
+        udata = {}
+        for k, v in data.iteritems():
+            try:
+                udata[k.encode('utf-8')] = unicode(v).encode('utf-8')
+            except UnicodeDecodeError:
+                udata[k.encode('utf-8')] = unicode(v, 'utf-8').encode('utf-8')
+        data = urlencode(udata)
 
     if params is not None:
         enc_params = urlencode(params, doseq=True)
@@ -132,7 +140,7 @@ def make_request(method, url,
             url = '%s&%s' % (url, enc_params)
         else:
             url = '%s?%s' % (url, enc_params)
-    
+
     resp, content = http.request(url, method, headers=headers, body=data)
 
     # Format httplib2 reqeusts as reqeusts objects
@@ -268,7 +276,16 @@ class ListResource(Resource):
 
     def get_instances(self, params=None, page=None, page_size=None):
         """
-        Query the list resource for a list of InstanceResources
+        Query the list resource for a list of InstanceResources.
+
+        Raises a TwilioRestException if requesting a page of results that does
+        not exist.
+
+        :param dict params: List of URL parameters to be included in request
+        :param int page: The page of results to retrieve (most recent at 0)
+        :param int page_size: The number of results to be returned.
+
+        :returns: -- the list of resources
         """
         params = params or {}
 
@@ -289,7 +306,7 @@ class ListResource(Resource):
         """
         Create an InstanceResource via a POST to the List Resource
 
-        :param dict body: Dictoionary of POST data
+        :param dict body: Dictionary of POST data
         """
         resp, instance = self.request("POST", self.uri, data=body)
 
@@ -635,7 +652,29 @@ class Calls(ListResource):
                if_machine=None, send_digits=None, timeout=None,
                application_sid=None):
         """
-        Make a phone call to a number
+        Make a phone call to a number.
+
+        :param string to: The phone number to call
+        :param string `from_`: The caller ID (must be a verified Twilio number)
+        :param string url: The URL to read TwiML from when the call connects
+        :param method: The HTTP method Twilio should use to request the url
+        :type method: None (defaults to 'POST'), 'GET', or 'POST'
+        :param string fallback_url: A URL that Twilio will request if an error occurs requesting or executing the TwiML at url.
+        :param string fallback_method: The HTTP method that Twilio should use to request the fallback_url
+        :type fallback_method: None (will make a 'POST' request), 'GET', or 'POST'
+        :param string status_callback: A URL that Twilio will request when the call ends to notify your app.
+        :param string status_method: The HTTP method Twilio should use when requesting the above URL.
+        :param string if_machine: Tell Twilio to try and determine if a machine
+            (like voicemail) or a human has answered the call.
+            See more in our `answering machine documentation
+            <http://www.twilio.com/docs/api/rest/making_calls#handling-outcomes-answering-machines">`_.
+        :type if_machine: None, 'Continue', or 'Hangup'
+        :param string send_digits: A string of keys to dial after connecting to the number.
+        :type send_digits: None or any combination of (0-9), '#', '*' or 'w' (to insert a half second pause).
+        :param int timeout: The integer number of seconds that Twilio should allow the phone to ring before assuming there is no answer.
+        :param string application_sid: The 34 character sid of the application Twilio should use to handle this phone call. Should not be used in conjunction with the url parameter.
+
+        :return: A :class:`Call` object
         """
         params = transform_params({
             "To": to,
@@ -662,7 +701,7 @@ class Calls(ListResource):
         return self.update_instance(sid, params)
 
     def cancel(self, sid):
-        """ If this call is queued or ringing, cancel the call
+        """ If this call is queued or ringing, cancel the call.
         Will not affect in-progress calls.
 
         :param sid: A Call Sid for a specific call
@@ -671,9 +710,8 @@ class Calls(ListResource):
         return self.update(sid, status=Call.CANCELED)
 
     def hangup(self, sid):
-        """ If this call is currenlty active, hang up the call.
-        If this call is scheduled to be made, remove the call
-        from the queue
+        """ If this call is currently active, hang up the call. If this call is
+        scheduled to be made, remove the call from the queue.
 
         :param sid: A Call Sid for a specific call
         :returns: Updated :class:`Call` resource
@@ -728,7 +766,7 @@ class CallerIds(ListResource):
             "PhoneNumber": phone_number,
             "FrienldyName": friendly_name,
             })
-        return self.get_instances(params=params, **kwargs) 
+        return self.get_instances(params=params, **kwargs)
 
     def update(self, sid, friendly_name=None):
         """
@@ -776,16 +814,31 @@ class CallerIds(ListResource):
 
 class PhoneNumber(InstanceResource):
 
-    def trasfer(self, account_sid):
+    def load(self, entries):
+        """ Set the proper Account owner of this phone number """
+
+        # Only check if entries has a uri
+        if "account_sid" in entries:
+            # Parse the parent's uri to get the scheme and base
+            uri = re.sub(r'AC(.*)', entries["account_sid"],
+                self.parent.base_uri)
+
+            self.parent = PhoneNumbers(uri, self.parent.auth)
+            self.base_uri = self.parent.uri
+
+        super(PhoneNumber, self).load(entries)
+
+    def transfer(self, account_sid):
         """
         Transfer the phone number with sid from the current account to another
         identified by account_sid
         """
-        pass
+        a = self.parent.transfer(self.name, account_sid)
+        self.load(a.__dict__)
 
     def update(self, **kwargs):
         """
-        Update this phone number instance
+        Update this phone number instance.
         """
         a = self.parent.update(self.name, **kwargs)
         self.load(a.__dict__)
@@ -842,7 +895,8 @@ class PhoneNumbers(ListResource):
                  status_callback_url=None, status_callback_method=None,
                  sms_url=None, sms_method=None, sms_fallback_url=None,
                  sms_fallback_method=None, voice_caller_id_lookup=None,
-                 account_sid=None, application_sid=None):
+                 account_sid=None, sms_application_sid=None,
+                 voice_application_sid=None, friendly_name=None):
         """
         Attempt to purchase the specified number. The only required parameters
         are **either** phone_number or area_code
@@ -863,7 +917,9 @@ class PhoneNumbers(ListResource):
                 "StatusCallbackMethod": status_callback_method,
                 "VoiceCallerIdLookup": voice_caller_id_lookup,
                 "AccountSid": account_sid,
-                "ApplicationSid": application_sid,
+                "SmsApplicationSid": sms_application_sid,
+                "VoiceApplicationSid": voice_application_sid,
+                "FriendlyName": friendly_name,
                })
 
         if phone_number:
@@ -887,19 +943,20 @@ class PhoneNumbers(ListResource):
         """
         return self.available_phone_numbers.list(**kwargs)
 
-    def trasfer(self, sid, account_sid):
+    def transfer(self, sid, account_sid):
         """
         Transfer the phone number with sid from the current account to another
         identified by account_sid
         """
-        return self.update_instance(sid, {"Url": url, "Method": method})
+        return self.update(sid, account_sid=account_sid)
 
     def update(self, sid, api_version=None, voice_url=None, voice_method=None,
                voice_fallback_url=None, voice_fallback_method=None,
                status_callback_method=None, sms_url=None, sms_method=None,
                sms_fallback_url=None, sms_fallback_method=None,
                voice_caller_id_lookup=None, account_sid=None,
-               application_sid=None):
+               voice_application_sid=None, status_callback=None,
+               application_sid=None, sms_application_sid=None):
         """
         Update this phone number instance
         """
@@ -909,14 +966,16 @@ class PhoneNumbers(ListResource):
                 "VoiceMethod": voice_method,
                 "VoiceFallbackUrl": voice_fallback_url,
                 "VoiceFallbackMethod": voice_fallback_method,
+                "VoiceApplicationSid": voice_application_sid or application_sid,
+                "StatusCallback": status_callback,
                 "StatusCallbackMethod": status_callback_method,
                 "SmsUrl": sms_url,
                 "SmsMethod": sms_method,
                 "SmsFallbackUrl": sms_fallback_url,
                 "SmsFallbackMethod": sms_fallback_method,
+                "SmsApplicationSid": sms_application_sid or application_sid,
                 "VoiceCallerIdLookup": voice_caller_id_lookup,
                 "AccountSid": account_sid,
-                "ApplicationSid": application_sid,
                 })
         return self.update_instance(sid, params)
 
@@ -947,7 +1006,7 @@ class Sandboxes(ListResource):
         """
         Update your Twilio Sandbox
         """
-        data = transform_params({
+        body = transform_params({
                 "VoiceUrl": voice_url,
                 "VoiceMethod": voice_method,
                 "SmsUrl": sms_url,
@@ -986,10 +1045,11 @@ class SmsMessages(ListResource):
         """
         Create and send a SMS Message.
 
-        :param to: The destination phone number.
-        :param from_: The phone number sending this message.
-        :param body: The message you want to send, limited to 160 characters.
-        :param status_callback: URL that Twilio will update with message info
+        :param string to: The destination phone number.
+        :param string `from_`: The phone number sending this message (must be a verified Twilio number)
+        :param string body: The message you want to send, limited to 160 characters.
+        :param status_callback: A URL that Twilio will POST to when your message is processed.
+        :param string application_sid: The 34 character sid of the application Twilio should use to handle this phone call.
         """
         params = transform_params({
             "To": to,
@@ -1001,21 +1061,27 @@ class SmsMessages(ListResource):
         return self.create_instance(params)
 
     @normalize_dates
-    def list(self, to=None, from_=None, before=None, after=None, **kwargs):
+    def list(self, to=None, from_=None, before=None, after=None,
+             date_sent=None, **kwargs):
         """
         Returns a page of :class:`SMSMessage` resources as a list. For
         paging informtion see :class:`ListResource`.
 
         :param to: Only show SMS messages to this phone number.
-        :param from_: Onlye show SMS message from this phone number.
+        :param from_: Only show SMS messages from this phone number.
+        :param date after: Only list SMS messages sent after this date.
+        :param date before: Only list SMS message sent before this date.
+        :param date date_sent: Only list SMS message sent on this date.
+        :param `from_`: Only show SMS messages from this phone number.
         :param date after: Only list recordings logged after this datetime
-        :param date before: Only list recordings logger before this datetime
+        :param date before: Only list recordings logged before this datetime
         """
         params = transform_params({
             "To": to,
             "From": from_,
             "DateSent<": before,
             "DateSent>": after,
+            "DateSent": parse_date(date_sent),
             })
         return self.get_instances(params=params, **kwargs)
 
@@ -1035,7 +1101,7 @@ class ShortCodes(ListResource):
     def list(self, short_code=None, friendly_name=None, **kwargs):
         """
         Returns a page of :class:`ShortCode` resources as a list. For
-        paging informtion see :class:`ListResource`.
+        paging information see :class:`ListResource`.
 
         :param short_code: Only show the ShortCode resources that match this
                            pattern. You can specify partial numbers and use '*'
@@ -1052,8 +1118,7 @@ class ShortCodes(ListResource):
     def update(self, sid, friendly_name=None, api_version=None, url=None,
                method=None, fallback_url=None, fallback_method=None):
         """
-        Returns a page of :class:`SMSMessage` resources as a list. For
-        paging informtion see :class:`ListResource`.
+        Update a specific :class:`ShortCode`, by specifying the sid.
 
         :param friendly_name: Description of the short code, with maximum
                               length 64 characters.
@@ -1185,10 +1250,10 @@ class Conferences(ListResource):
             "FriendlyName": friendly_name,
             "DateUpdated<": updated_before,
             "DateUpdated>": updated_after,
-            "DateUpdated=": parse_date(updated),
+            "DateUpdated": parse_date(updated),
             "DateCreated<": created_before,
             "DateCreated>": created_after,
-            "DateCreated=": parse_date(created),
+            "DateCreated": parse_date(created),
             })
         return self.get_instances(params=params, **kwargs)
 
@@ -1233,7 +1298,7 @@ class Applications(ListResource):
                sms_url=None, sms_method=None, sms_fallback_url=None,
                sms_fallback_method=None, sms_status_callback=None):
         """
-        Update an :class:`Application` with any of these optional parameters.
+        Create an :class:`Application` with any of these optional parameters.
 
         :param friendly_name: A human readable description of the application,
                               with maximum length 64 characters.
@@ -1285,6 +1350,8 @@ class Applications(ListResource):
                 "StatusCallback": status_callback,
                 "StatusCallbackMethod": status_callback_method,
                 "VoiceCallerIdLookup": voice_caller_id_lookup,
+                "SmsUrl": sms_url,
+                "SmsMethod": sms_method,
                 "SmsFallbackUrl": sms_fallback_url,
                 "SmsFallbackMethod": sms_fallback_method,
                 "SmsStatusCallback": sms_status_callback,
@@ -1312,6 +1379,8 @@ class Applications(ListResource):
                 "StatusCallback": status_callback,
                 "StatusCallbackMethod": status_callback_method,
                 "VoiceCallerIdLookup": voice_caller_id_lookup,
+                "SmsUrl": sms_url,
+                "SmsMethod": sms_method,
                 "SmsFallbackUrl": sms_fallback_url,
                 "SmsFallbackMethod": sms_fallback_method,
                 "SmsStatusCallback": sms_status_callback,
